/************************************************************************
 * Purpose:     To send same message to all other processes and then 
 *              receive a message of same length from all other 
 *              processes.
 *
 * Author:      Samir Mehta
 * Date:        2/29/2000
 *
 ************************************************************************/

#include <stdio.h>
#include <mpi.h>

#define LENGTH 100000
#define INCR 10000


void main ( int argc, char *argv[] )
{
  int ierr, rank, size, i;
  int length, tag;
  MPI_Status status;
  double start, finish, time;
  float message[LENGTH];

  ierr = MPI_Init(&argc, &argv);
  ierr = MPI_Comm_size(MPI_COMM_WORLD, &size);
  ierr = MPI_Comm_rank(MPI_COMM_WORLD, &rank);

  if (rank == 0) 
  {
    printf("Length\tTime\t (for %d processes)\n",size);

    for(length = INCR; length <= LENGTH; length += INCR)
    {

      start = MPI_Wtime();

      tag = 100;

      for (i = 1; i < size; i++)
      {
        ierr = MPI_Ssend(message, length, MPI_FLOAT, i, tag, MPI_COMM_WORLD);

        ierr = MPI_Recv(message, length, MPI_FLOAT, i, tag + 1, MPI_COMM_WORLD, &status);
      }

      finish = MPI_Wtime();
    
      time = finish - start;

      printf("%d\t%f\n", length, time);
    }

  }

  else 
  {

    for(length = INCR; length <= LENGTH; length += INCR)
    {

      tag = 100;

      ierr = MPI_Recv(message, length, MPI_FLOAT, 0, tag, MPI_COMM_WORLD, &status);

      ierr = MPI_Ssend(message, length, MPI_FLOAT, 0, tag + 1, MPI_COMM_WORLD);
    }

  }

  MPI_Finalize();

}
